/* $Id$ */
/* ----------------------------------------------------------------------- *
 *   
 *   Copyright 2003 H. Peter Anvin - All Rights Reserved
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, Inc., 53 Temple Place Ste 330,
 *   Bostom MA 02111-1307, USA; either version 2 of the License, or
 *   (at your option) any later version; incorporated herein by reference.
 *
 * ----------------------------------------------------------------------- */

/*
 * cfplayer.v
 * 
 * Top-level for sound-playing mechanism
 * 
 * Plays raw CD-format music written to CompactFlash (without partition
 * table or filesystem)
 */

module cfplayer (
		 clkin,
		 
		 cf_a,
                 cf_d,
                 cf_rdy,
                 cf_wait_n,
                 cf_ce1_n,
                 cf_ce2_n,
                 cf_oe_n,
                 cf_we_n,
                 cf_reg_n,

		 audio_l,
		 audio_r,

		 led,
		 s7_0,
		 s7_1,
		 );
   
   input         clkin;
   
   output [10:0] cf_a;
   inout [15:0]  cf_d;
   input 	 cf_rdy;
   input 	 cf_wait_n;
   output 	 cf_ce1_n;
   output 	 cf_ce2_n;
   output 	 cf_oe_n;
   output 	 cf_we_n;
   output 	 cf_reg_n;
   
   output 	 audio_l;
   output 	 audio_r;

   output [7:0]  led;
   output [7:0]  s7_0;
   output [7:0]  s7_1;

   wire [15:0] 	 audio_q;
   wire 	 data_ready_n;
   reg [11:0] 	 audio_data_l = 0;
   reg [11:0] 	 audio_data_r = 0;
   reg 		 rdreq;

   wire 	 audio_strobe;
   reg 		 audio_strobe1;
   
   wire 	 audio_clk;
   wire 	 cf_clk;

   wire [7:0] 	 s7_val;
   
   assign 	 led[0] = cf_rdy;
   assign 	 led[1] = cf_wait_n;
   assign 	 led[2] = audio_data_l[11];
   assign 	 led[3] = audio_data_r[11];
   assign 	 led[4] = audio_l;
   assign 	 led[5] = audio_r;
   assign 	 led[6] = audio_strobe;
   assign 	 led[7] = data_ready_n;

   // Primary PLL for design
   pll1	pll1_inst (
		   .inclk0 ( clkin ),	// 50 MHz clkin
		   .pllena ( 1 ),	// PLL enable
		   .areset ( 0 ),	// PLL reset
		   .c0 ( audio_clk ), 	// Audio clock - 200 MHz
		   .c1 ( cf_clk )       // CompactFlash clock - 12.5 MHz
	);

   // CompactFlash read unit
   cfstream cfstream_inst (
			   .clk ( cf_clk ),
			   .cf_a ( cf_a ),
			   .cf_d ( cf_d ),
			   .cf_bsy_n ( cf_rdy ),
			   .cf_wait_n ( cf_wait_n ),
			   .cf_ce1_n ( cf_ce1_n ),
			   .cf_ce2_n ( cf_ce2_n ),
			   .cf_oe_n ( cf_oe_n ),
			   .cf_we_n ( cf_we_n ),
			   .cf_reg_n ( cf_reg_n ),
			   .q ( audio_q ),
			   .data_ready_n ( data_ready_n ),
			   .rdreq ( rdreq ),
			   .bsy_time ( s7_val )
			   );

   // We need to advance the FIFO exactly one datum for each
   // time the audio core latches a datum.
   always @( posedge cf_clk )
     begin
	rdreq <= ( audio_strobe & ~audio_strobe1 & ~data_ready_n );
	audio_strobe1 <= audio_strobe;

	if ( rdreq )
	  begin
	     // Careful here - get the endianism, signedness and
	     // LSB truncation right
	     audio_data_l[11]   <= ~audio_q[7];	// Signed data
	     audio_data_l[10:4] <= audio_q[6:0];
	     audio_data_l[3:0]  <= audio_q[15:12];
	     audio_data_r[11]   <= ~audio_q[7];	// Signed data
	     audio_data_r[10:4] <= audio_q[6:0];
	     audio_data_r[3:0]  <= audio_q[15:12];
	  end
     end

   always @( s7_val )
     begin
	case ( s7_val[7:4] )
	  4'h0: s7_1 = ~8'b00111111;
	  4'h1: s7_1 = ~8'b00000110;
	  4'h2: s7_1 = ~8'b01011011;
	  4'h3: s7_1 = ~8'b01001111;
	  4'h4: s7_1 = ~8'b01100110;
	  4'h5: s7_1 = ~8'b01101101;
	  4'h6: s7_1 = ~8'b01111101;
	  4'h7: s7_1 = ~8'b00000111;
	  4'h8: s7_1 = ~8'b01111111;
	  4'h9: s7_1 = ~8'b01101111;
	  4'hA: s7_1 = ~8'b01110111;
	  4'hB: s7_1 = ~8'b01111100;
	  4'hC: s7_1 = ~8'b00111001;
	  4'hD: s7_1 = ~8'b01011110;
	  4'hE: s7_1 = ~8'b01111001;
	  4'hF: s7_1 = ~8'b01110001;
	endcase
	case ( s7_val[3:0] )
	  4'h0: s7_0 = ~8'b00111111;
	  4'h1: s7_0 = ~8'b00000110;
	  4'h2: s7_0 = ~8'b01011011;
	  4'h3: s7_0 = ~8'b01001111;
	  4'h4: s7_0 = ~8'b01100110;
	  4'h5: s7_0 = ~8'b01101101;
	  4'h6: s7_0 = ~8'b01111101;
	  4'h7: s7_0 = ~8'b00000111;
	  4'h8: s7_0 = ~8'b01111111;
	  4'h9: s7_0 = ~8'b01101111;
	  4'hA: s7_0 = ~8'b01110111;
	  4'hB: s7_0 = ~8'b01111100;
	  4'hC: s7_0 = ~8'b00111001;
	  4'hD: s7_0 = ~8'b01011110;
	  4'hE: s7_0 = ~8'b01111001;
	  4'hF: s7_0 = ~8'b01110001;
	endcase
     end
   
   sound sound_inst (
		     .clk ( audio_clk ),
		     // Careful here -- remember CD data is bigendian
		     // The sound module takes 12-bit inputs; throw away
		     // 4 LSB
		     .data_l ( audio_data_l ),
		     .data_r ( audio_data_r ),
		     .audio_l ( audio_l ),
		     .audio_r ( audio_r ),
		     .audio_clk ( audio_strobe ),
		     );

endmodule // cfplayer
